package com.sf.day22._03_class.test;

import com.sf.day22._03_class.dt.Cat;
import com.sf.day22._03_class.dt.Student;
import com.sf.day22._03_class.dt.User;
import com.sf.day22._03_class.util.PrintJavaBeanUtil;
import org.dom4j.Attribute;
import org.dom4j.Document;
import org.dom4j.DocumentException;
import org.dom4j.Element;
import org.dom4j.io.SAXReader;
import org.junit.Test;

import java.io.InputStream;
import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;

public class ClassTest {


    // 我的需求是这个方法传入什么对象就打印什么对象
    //
    @Test
    public void test(){
        User user = new User("zhangsan",10);
        Cat cat = new Cat("xiaomi","黄色",10);
        PrintJavaBeanUtil.print(cat);
    }

    /**
     * 获取字节码的三种方式
     * 第一种方式 Class.forName("全限定名") com.sf.day22._03_class.test.ClassTest
     * 第二种方式 Person.class    获取字节码
     * 第三种方式 Object 当中有一个方法叫做getClass()  能够获取字节码信息
     */

    @Test
    public void test1() throws ClassNotFoundException {
        // 1 创建 字节码对象
        Class<?> clz = Class.forName("com.sf.day22._03_class.dt.Cat");
        System.out.println(clz);
        // 2 调用是类名.class     在Object 当中有这样class 的字段
        Class<Cat> clz1 = Cat.class;
        System.out.println(clz1  == clz);
        // 3 使用对象方式获取class
        Cat cat = new Cat();
        Class<? extends Cat> clz2 = cat.getClass();
        System.out.println(clz2);

        //选择 ： 你如果有对象那就用getClass() ，
        // 如果此时只有类 调用.class
        // 如果你能够获取全限定名那用Class.forName
    }

    @Test
    public void test2() throws NoSuchMethodException {
        // 获取构造器
        // 1 获取字节码对象
        Class<Student> clz = Student.class;
        // 2 通过字节码对象获取构造器
        // 获取所有public 构造器
//        Constructor<?>[] constructors = clz.getConstructors();
        // 如果你想要获取所有的构造器包含非public privte
//        Constructor<?>[] declaredConstructors = clz.getDeclaredConstructors();
//
//        for (Constructor<?> constructor : declaredConstructors) {
//            System.out.println(constructor.getName());
//            System.out.println(constructor.getParameterCount());
//        }
        // 我就想要获取 Student(id,name,age)
//        Constructor<Student> constructor = clz.getConstructor(Long.class, String.class,Integer.class);
        Constructor<Student> declaredConstructor = clz.getDeclaredConstructor(Long.class, String.class, Integer.class);
        System.out.println(declaredConstructor.getName());
        System.out.println(declaredConstructor.getParameterCount());
    }


    @Test
    public void test3() throws NoSuchMethodException, IllegalAccessException, InstantiationException, InvocationTargetException {
        // 创建对象 迪奥用Student 全参构造器
        //1 获取字节码对象
        Class<Student> clz = Student.class;
        //2 通过字节码对象获取构造器
//        Constructor<Student> constructor = clz.getConstructor(Long.class, String.class, Integer.class, Integer.class);
//        //3 创建对象 new Instance(..);
//        Student student = constructor.newInstance(1L, "zhangsan", 10, 30);
//        System.out.println(student);
        // 这种是一种省略写法 ， 正常要先获取构造器在newInstance() 如果调用无参构造器clz.newInstance();
        Student student = clz.newInstance();
    }

    /**
     * 需求: 模拟spring 底层源码实现
     * 步骤: 1 创建一个xml 文件, 在xml 文件中定义一个beans根标签
     *      2 在根标签中定义 bean 标签里面有一个 class属性值为 = com.sf.day22._03_class.dt.Student
     *       Element element1 = elements.get(1);
     *       Attribute attribute = element1.attribute("属性名");
     *       attribute.getValue() 可以获取值
     *      3 要求创建出来Student 对象
     *      4 如果把class 的值修改成com.sf.day22._03_class.dt.Cat
     *      要创建Cat 对象java 不可以改动只能改动配置文件
     *
     *
     */
    @Test
    public void test4() throws DocumentException, ClassNotFoundException, IllegalAccessException, InstantiationException {
        //读取Xml文件信息
        InputStream inputStream = Thread.currentThread().
                getContextClassLoader().
                getResourceAsStream("spring.xml");
        // XARReader
        SAXReader saxReader = new SAXReader();
        Document document = saxReader.read(inputStream);
        //获取根元素
        Element element = document.getRootElement();
        // 通过跟元素获取bean
        Element bean = element.element("bean");
        // 获取bean 标签属性
        Attribute attribute = bean.attribute("class");
        String value = attribute.getValue();
        System.out.println(value);

        // 通过反射创建出来Student 对象
        Class<?> clz = Class.forName(value);
        Object o = clz.newInstance();
        System.out.println(o);
    }

    /**
     * 通过字节码获取方法
     */
    @Test
    public void test5() throws NoSuchMethodException, IllegalAccessException, InstantiationException, InvocationTargetException {
        //1 获取字节码对象
        Class<Student> clz = Student.class;
        //2 通过字节吗获取方法
        // getMethods() 获取所有public 方法, 还包含父类方法
//        Method[] methods = clz.getMethods();
        // getDeclaredMethods 获取的是当前类的所有方法包含非public
//        Method[] methods = clz.getDeclaredMethods();
//        for (Method method : methods) {
//            System.out.println("方法的名字"+ method.getName());
//            System.out.println("方法的参数"+method.getParameterCount());
//        }
        /**
         * 第一个参数: 方法名字
         * 第二个参数: 方法参数的类型
         */
        Method method = clz.getMethod("m1");
//        System.out.println(method.getName());
//        System.out.println("参数数量"+ method.getParameterCount());

        // 获取eat 方法呢?
//        Method method1 = clz.getDeclaredMethod("eat");
//        System.out.println(method1);

        // 调用run 方法
        /**
         * invoke 方法当中有两个参数
         * 第一个参数: 对象
         * 第二个参数: 你调用方法传递的参数
         * publci void  run(String shpe)
         */
//        method.invoke(clz.newInstance(),"耐克");
        // 如果调用的是静态方法,第一个参数是可以不用传递的
        method.invoke(null);
        //作用:
        // 需求: 有一个类
        /**
         * class Common{
         *     private String name = "123"
         *     public String getName(){
         *         return name;
         *     }
         * }
         *
         * 在测试类中已知条件
         * String  str = 全限定名
         * String  methodName = "name"
         * 要求调用getName 获取返回值123
         */
    }

}
