package com.sf.day23._01_javaBean;

import org.junit.Test;

import java.beans.BeanInfo;
import java.beans.IntrospectionException;
import java.beans.Introspector;
import java.beans.PropertyDescriptor;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.HashMap;
import java.util.Map;

public class LombokTest {

    @Test
    public void test(){
        Person person = new Person();
        person.setAge(10);
        System.out.println(person.getAge());
    }

    @Test
    public void test1() throws ClassNotFoundException, IllegalAccessException, InstantiationException, NoSuchMethodException, InvocationTargetException {
        /**
         *  已知条件有一个类全名 com.sf.day23._01_javaBean.Student
         *  已知属性名   name
         *  要求获取name 的值其实就是要调用getName方法获取值
         *
         *  使用反射如何操作?
         */
        // 我们现在目标要获取对象当中getName返回值,也就是说我要调用getName 方法
        String className = "com.sf.day23._01_javaBean.Student";
        String propertyName= "name";

        // 1 获取字节码对象
        Class<Student> clz = (Class<Student>) Class.forName(className);
        // 2 通过字节码对象创建实力
        Student student = clz.newInstance();
        // 3 获取getName 方法
        //  这个第三方如何获取getName 方法 get + N + amme
        String methodName = "get"+ propertyName.substring(0,1).toUpperCase() +
                propertyName.substring(1);
        System.out.println(methodName);
        // get+ 获取name 的第一个字符然后把他转换成大写 , 获取剩余内容进行拼接
        // 4 通过反射去调用方法
        Method method = clz.getMethod(methodName);
        // 5 调用方法
        System.out.println(method.invoke(student));

    }

    @Test
    public void test2() throws IntrospectionException, IllegalAccessException, InstantiationException, InvocationTargetException {
        // 获取对象的属性
        // 内省是在反射的基础上去进行操作的, 也是要用到我们的字节吗对象
        // 1 需要获取到字节码信息
        Class<Person> clas = Person.class;
        // 2 获取javaBean 信息
        // 现在我们的javaBean 信息就已经是封装到了beanInfo 当中了
        // 这在获取javaBean 信息的时候他不仅会获取本来当中属性还会获取他父类的属性 Object class
        BeanInfo beanInfo = Introspector.getBeanInfo(clas,Object.class);
        // 3 在beanInfo 当中有一个方法可以获取到我们javaBean 当中属性信息
        // 由于我们javaBean 当中可能存在着多个属性,所以返回是一个数组
        // 每一个属性比如name 他都会封装到PropertyDescriptor 当中 比如我们属性name 我们属性类型
        PropertyDescriptor[] descriptors = beanInfo.getPropertyDescriptors();
        // 4 遍历拿到每一个属性描述器
        // 注意: 这个属性描述器当中是包含了咱们属性的所有信息 名字 类型...
        for (PropertyDescriptor propertyDescriptor :descriptors){
            System.out.println("属性名字---" +propertyDescriptor.getName());
            System.out.println("属性的类型---"+propertyDescriptor.getPropertyType() );
            // 一个getReadMethod 相当于get属性名   getWriteMethod 相当于set方法
            // 还记得我们在拼接getName 方法的时候有多麻烦吗? 需要不断调用字符串方法进行转换
            // 需求: 如果属性的name 是name , 我们就给他的值设置为zhangsan
            if("name".equals(propertyDescriptor.getName())){
                // set 方法
                Person person = clas.newInstance();
                Method method = propertyDescriptor.getWriteMethod();
                method.invoke(person,"zhangsan");
                // 获取name 内容
                //get 方法
                System.out.println(propertyDescriptor.getReadMethod().invoke(person));
            }
        }
    }

    // Javabean 转 map
    // 要求提供一个方法, 方法传递一个对象,对象的类型不固定, 要求返回一个map
    // 比如我传递是一个User(id,name,sn) id=1,name=zhangsan,sn=123   返回map{id=1,name=zhangsan,sn=123}
    // 如果我们传递的Cat(color,name)   返回的就是 map{color=黄色,name=小黄}
    public Map<String,Object> beanToMap(Object object) throws IntrospectionException, IllegalAccessException, InstantiationException, InvocationTargetException {
        // 创建map
        Map<String,Object> map = new HashMap<>();
        // 最终 要把javaBean 当中属性 放到map
        // 以为这我要获取javaBean 属性,如果我能获取到, 放到map 当中put
        // 问题转移到了如何获取javaBean 属性 -> 内省
        Class<?> clz = object.getClass();
        // 1 获取beanInfo 信息
        BeanInfo beanInfo = Introspector.getBeanInfo(clz, Object.class);
        // 2 根据beanInfo 获取到类当中所有的属性描述器
        PropertyDescriptor[] propertyDescriptors = beanInfo.getPropertyDescriptors();
        // 3 遍历所有的属性描述器拿到每一个属性描述器
        for (PropertyDescriptor propertyDescriptor : propertyDescriptors) {
            // 拿到属性描述起 我就可以拿到属性的名字, 我也可以拿到属性的值
            // 有属性的名字和属性的值 我们不就可以往map 当中去存储数据了吗
            String name = propertyDescriptor.getName();
            // 获取属性值如何获取?  调用get方法去获取  , 在属性名描述起中 getReadMethod();
            Object val = propertyDescriptor.getReadMethod().invoke(object);
            // 往map 当中存储数据
            map.put(name,val);
        }
        return map;
    }

    @Test
    public void test3() throws InvocationTargetException, IntrospectionException, InstantiationException, IllegalAccessException {
        Map<String, Object> map = beanToMap(new Student("lisi"));
        System.out.println(map);
    }


    /**
     * 把Map 转成JavaBean
     * 传递Map<String,Object>  map(id=1,name=zhangsan,age=10)   Person()
     *                        map(name="xiaoming")             Student();
     *
     *  需求: 把Map 转成JavaBean 对象
     *  public Object mapTojavaBean(Map<String,Object) map, Class clz>)
     */
    @Test
    public void test4() throws InvocationTargetException, IntrospectionException, InstantiationException, IllegalAccessException {
        Map<String,Object> map = new HashMap<>();
        map.put("name","zhangsan");
        // 现在功能已经是实现完了,有一点缺陷我们返回值是一个Object 对象需要进行强转
        // 我想要的效果我们传入了什么类型 返回值就是什么类型, 这里就要用到我们之前学习过泛型
        Person person = mapToJavaBean(map, Person.class);
        System.out.println(person);
    }

    public <T>T mapToJavaBean(Map<String,Object> map,Class<T> clz) throws IllegalAccessException, InstantiationException, IntrospectionException, InvocationTargetException {
        // 把map 准成javaBean
        // 把map 当中数据取出来, 存到javaBean
        // 把map 当中数据取出来,可以通过遍历方式拿到每一个key-value
        // 把数据存到javaBean属性中,  使用内省getWriteMethod():
        T t = clz.newInstance();

        // 使用内省
        BeanInfo beanInfo = Introspector.getBeanInfo(clz, Object.class);
        // id name age
        PropertyDescriptor[] propertyDescriptors = beanInfo.getPropertyDescriptors();
        /**
         *  key   value
         *  id     1
         *  name   zhangsan
         *  age    10
         *
         *  Student(1,zhangsan,10)
         */
        for (PropertyDescriptor propertyDescriptor : propertyDescriptors) {
            // 对象当中属性名
            // map 当中key - 属性名 , map 当中value 属性值
            String key = propertyDescriptor.getName();
            Object val = map.get(key);
            // 最终要把value 存储对象的字段中 要调用set 方法 getWriteMethod
            propertyDescriptor.getWriteMethod().invoke(t,val);
        }
        return t;
    }
}
