package com.sf.day24._02_method_ref;

import com.sf.day24._02_method_ref.dt.Product;
import com.sf.day24._02_method_ref.servcie.FuntionService;
import org.junit.Test;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.function.Consumer;
import java.util.function.Function;
import java.util.function.Supplier;

public class MethodRefrenceTest {


    @Test
    public void test(){
        // 拿遍历集合去体验方法引用
        // 使用forEach 遍历集合
        List<String> list = Arrays.asList("123","456","789");
        list.forEach(new Consumer<String>() {
            // 这个s 我们集合当中元素
            @Override
            public void accept(String s) {
                System.out.println(s);
            }
        });

        // 使用lambda 表达式
        list.forEach(s -> System.out.println(s));
        // 使用方法引用方式
        list.forEach(System.out::println);
    }

    //静态方法 的 方法引用语法
    @Test
    public void test1(){
        Supplier supplier = new Supplier() {
            @Override
            public Object get() {
                return Math.random();
            }
        };


        //使用lambda 表达式去简化上面写法
        Supplier supplier1 = () -> Math.random();
        //使用方法引用
        Supplier supplier2 = Math::random;
        Object o = supplier2.get();
        System.out.println(o);
    }

    /**
     * 需求: 利用Supplier 接口 , 在方法中返回new Product
     * 分别使用匿名内部类防腐蚀
     * 使用lombda 表达式方式
     * 使用方法引用方法
     *
     * 针对于构造器 简化类名 :: new
     *
     */
    @Test
    public void test4(){
        Supplier<Product> supplier = new Supplier<Product>() {
            @Override
            public Product get() {
                return new Product();
            }
        };

        // 使用lambda 表达式去简化
        Supplier<Product> supplier1 = () -> new Product();
        // 使用方法引用方式进行简化
        Supplier<Product> supplier2 = Product::new;
        System.out.println(supplier2);
    }

    /**
     * 需求: 利用Function 接口方法 要查看我们传入字符串长度
     * 要求: 分别使用 匿名内部类  lambda 表达式  方法引用
     */
    @Test
    public void test5(){
        Function<String,Integer> function = new Function<String, Integer>() {
            @Override
            public Integer apply(String s) {
                return s.length();
            }
        };

        // 使用lambda 表达式简化
        Function<String,Integer> function1 = s -> s.length();
        // 使用方法引用方式简化
        Function<String,Integer> function2 = String::length;
        System.out.println(function2.apply("123123"));
    }
    /**
     * 需求
     * 要求提供一个函数式接口 里面有两个参数List<String> ,int size
     * 要求把长度大于3的字符串 放到一个新的集合中,这个新的集合大小size
     *
     * 要求分别使用匿名内部类 , lambda 表达式 , 方法引用去操作
     *
     */

    @Test
    public void test6(){
        // 使用匿名内部类方式
        FuntionService funtionService = new FuntionService() {
            @Override
            public List<String> fun(List<String> list, Integer size) {
                List<String> newList = new ArrayList<>();
                for (String item : list) {
                    if(item.length()>3){
                        if(newList.size() <size){
                            newList.add(item);
                        }
                    }
                }
                return newList;
            }
        };

        List<String> list = Arrays.asList("aaaaaaa","ssssssss", "ab", "a", "ssssss");
//        List<String> list1 = funtionService.fun(list, 2);
//        System.out.println(list1);

        // 使用Lambda 表达式进行优化
        FuntionService funtionService1 = (l,size)->{
            List<String> newList = new ArrayList<>();
            for (String item : list) {
                if(item.length()>3){
                    if(newList.size() <size){
                        newList.add(item);
                    }
                }
            }
            return newList;
        };

        List<String> stringList = funtionService1.fun(list, 2);
        System.out.println(stringList);

        // 方法引用如何使用
        // 方法作用简化咱们lambda 表达式方法体的写法, 若有有一个方法的内容和lambda 表达的方法体的内容
        // 相同的就可以使用方法引用语法
        FuntionService funtionService2 = MethodRefrenceTest::method11;
        List<String> list1 = funtionService2.fun(list, 2);
        System.out.println(list1);
    }

    public static List<String> method11(List<String> list, Integer size){
        List<String> newList = new ArrayList<>();
        for (String item : list) {
            if(item.length()>3){
                if(newList.size() <size){
                    newList.add(item);
                }
            }
        }
        return newList;
    }
}
