package com.sf.day27._01_jdbc_concat;

import com.sf.day27._01_jdbc_concat.domain.Integral;
import org.junit.Test;

import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.util.ArrayList;
import java.util.List;
import java.util.Scanner;

public class JDBCTest {


    /**
     * 使用JDBC 完成积分表新增操作
     */
    @Test
    public void test() throws Exception {
        //1 加载驱动类
        Class.forName("com.mysql.jdbc.Driver");
        //2 获取连接对象
        Connection connection = DriverManager.getConnection("jdbc:mysql:///jdbcdemo", "root", "admin");
        //3 创建语句对象
        String sql = "insert into integral(name,age,integral) values(?,?,?)";
        PreparedStatement preparedStatement = connection.prepareStatement(sql);
        preparedStatement.setString(1,"test");
        preparedStatement.setInt(2,10);
        preparedStatement.setInt(3,20);

        //4 执行sql 语句
        int count = preparedStatement.executeUpdate();
        // 判断是否插入成功如果插入成功我打印插入成功如果没插入成功抛出RuntimeException
        if(count>0){
            System.out.println("插入成功");
        }else{
            throw new RuntimeException("插入失败");
        }
        //5 释放资源
        connection.close();
        preparedStatement.close();
    }

    @Test
    public void test1() throws Exception {
        //1 加载驱动类
        Class.forName("com.mysql.jdbc.Driver");
        //2 获取连接对象
        Connection connection = DriverManager.getConnection("jdbc:mysql:///jdbcdemo", "root", "admin");
        //3 创建语句对象
        String sql = "delete from integral where id = ?";
        PreparedStatement preparedStatement = connection.prepareStatement(sql);
        preparedStatement.setLong(1,1L);

        //4 执行sql 语句
        int count = preparedStatement.executeUpdate();
        // 判断是否插入成功如果插入成功我打印插入成功如果没插入成功抛出RuntimeException
        if(count>0){
            System.out.println("插入成功");
        }else{
            throw new RuntimeException("插入失败");
        }
        //5 释放资源
        connection.close();
        preparedStatement.close();
    }

    @Test
    public void test2() throws Exception {
        //1 加载驱动类
        Class.forName("com.mysql.jdbc.Driver");
        //2 获取连接对象
        Connection connection = DriverManager.getConnection("jdbc:mysql:///jdbcdemo", "root", "admin");
        //3 创建语句对象
        String sql = "update integral set name=?,age=?,integral=? where id = ?";
        PreparedStatement preparedStatement = connection.prepareStatement(sql);
        preparedStatement.setString(1,"ssss");
        preparedStatement.setInt(2,30);
        preparedStatement.setInt(3,50);
        preparedStatement.setLong(4,2);

        //4 执行sql 语句
        int count = preparedStatement.executeUpdate();
        // 判断是否插入成功如果插入成功我打印插入成功如果没插入成功抛出RuntimeException
        if(count>0){
            System.out.println("插入成功");
        }else{
            throw new RuntimeException("插入失败");
        }
        //5 释放资源
        connection.close();
        preparedStatement.close();
    }

    // JDBC 查询单个 和查询所有
    @Test
    public void test3() throws Exception {
        //1 加载驱动类
        Class.forName("com.mysql.jdbc.Driver");
        //2 获取连接对象
        Connection connection = DriverManager.getConnection("jdbc:mysql:///jdbcdemo", "root", "admin");
        //3 创建语句对象
        String sql = "select * from integral where id = ?";
        PreparedStatement preparedStatement = connection.prepareStatement(sql);
        preparedStatement.setLong(1,2L);

        //4 执行sql 语句
        ResultSet resultSet = preparedStatement.executeQuery();
        if(resultSet.next()){
            Integral integral = new Integral();
            integral.setAge(resultSet.getInt("age"));
            integral.setName(resultSet.getString("name"));
            integral.setId(resultSet.getLong("id"));
            integral.setIntegral(resultSet.getInt("integral"));
            System.out.println(integral);
        }
        //5 释放资源
        connection.close();
        preparedStatement.close();
    }


    @Test
    public void test4() throws Exception {
        //1 加载驱动类
        Class.forName("com.mysql.jdbc.Driver");
        //2 获取连接对象
        Connection connection = DriverManager.getConnection("jdbc:mysql:///jdbcdemo", "root", "admin");
        //3 创建语句对象
        String sql = "select * from integral";
        PreparedStatement preparedStatement = connection.prepareStatement(sql);

        //4 执行sql 语句
        ResultSet resultSet = preparedStatement.executeQuery();
        List<Integral> list = new ArrayList<>();
        while (resultSet.next()){
            Integral integral = new Integral();
            integral.setAge(resultSet.getInt("age"));
            integral.setName(resultSet.getString("name"));
            integral.setId(resultSet.getLong("id"));
            integral.setIntegral(resultSet.getInt("integral"));

            list.add(integral);
        }
        System.out.println(list);
        //5 释放资源
        connection.close();
        preparedStatement.close();
    }
//    要求在控制台输入2 个数据最大值和最小值, 要求JDBC 查询出来指定范围的集合
    public static void main(String[] args) throws Exception {
        method3();
    }

    private static void method1() throws Exception {
        Scanner scanner = new Scanner(System.in);
        System.out.println("最大值:");
        int max = scanner.nextInt();
        System.out.println("最小值:");
        int min = scanner.nextInt();

        //1 加载驱动类
        Class.forName("com.mysql.jdbc.Driver");
        //2 获取连接对象
        Connection connection = DriverManager.getConnection("jdbc:mysql:///jdbcdemo", "root", "admin");
        //3 创建语句对象
        String sql = "select * from integral where integral between ? and ?";
        PreparedStatement preparedStatement = connection.prepareStatement(sql);
        preparedStatement.setInt(1,min);
        preparedStatement.setInt(2,max);
        //4 执行sql 语句
        ResultSet resultSet = preparedStatement.executeQuery();
        List<Integral> list = new ArrayList<>();
        while (resultSet.next()){
            Integral integral = new Integral();
            integral.setAge(resultSet.getInt("age"));
            integral.setName(resultSet.getString("name"));
            integral.setId(resultSet.getLong("id"));
            integral.setIntegral(resultSet.getInt("integral"));

            list.add(integral);
        }
        System.out.println(list);
        //5 释放资源
        connection.close();
        preparedStatement.close();

    }

//    要求按照积分进行排序, 查询出来数据集合

    private static void method2() throws Exception {

        //1 加载驱动类
        Class.forName("com.mysql.jdbc.Driver");
        //2 获取连接对象
        Connection connection = DriverManager.getConnection("jdbc:mysql:///jdbcdemo", "root", "admin");
        //3 创建语句对象
        String sql = "select * from integral order by integral";
        PreparedStatement preparedStatement = connection.prepareStatement(sql);
        //4 执行sql 语句
        ResultSet resultSet = preparedStatement.executeQuery();
        List<Integral> list = new ArrayList<>();
        while (resultSet.next()){
            Integral integral = new Integral();
            integral.setAge(resultSet.getInt("age"));
            integral.setName(resultSet.getString("name"));
            integral.setId(resultSet.getLong("id"));
            integral.setIntegral(resultSet.getInt("integral"));

            list.add(integral);
        }
        System.out.println(list);
        //5 释放资源
        connection.close();
        preparedStatement.close();

    }

//    查询名字为zhangsan 或者是lisi 或者是 王五的用户信息


    private static void method3() throws Exception {

        //1 加载驱动类
        Class.forName("com.mysql.jdbc.Driver");
        //2 获取连接对象
        Connection connection = DriverManager.getConnection("jdbc:mysql:///jdbcdemo", "root", "admin");
        //3 创建语句对象
        String sql = "select * from integral where name in ('zhangsan','lisi','wangwu')";
        PreparedStatement preparedStatement = connection.prepareStatement(sql);
        //4 执行sql 语句
        ResultSet resultSet = preparedStatement.executeQuery();
        List<Integral> list = new ArrayList<>();
        while (resultSet.next()){
            Integral integral = new Integral();
            integral.setAge(resultSet.getInt("age"));
            integral.setName(resultSet.getString("name"));
            integral.setId(resultSet.getLong("id"));
            integral.setIntegral(resultSet.getInt("integral"));

            list.add(integral);
        }
        System.out.println(list);
        //5 释放资源
        connection.close();
        preparedStatement.close();

    }
}
