package com.sf.day28.dao.impl;

import com.sf.day28.dao.IAccountDAO;
import com.sf.day28.util.JDBCUtil;

import java.math.BigDecimal;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;

public class AccountDAOImpl implements IAccountDAO {

    /**
     * 需求: 谁 name 转给 谁 name   amount
     * @param fromName  转账人名字
     * @param toName    收款人名字
     * @param amount    转账金额
     */
    @Override
    public void trans(String fromName, String toName, BigDecimal amount) {
        Connection connection = JDBCUtil.getConnection();
        PreparedStatement preparedStatement = null;


        try {
            /**
             * 我们出现 一个少了一个没有加 ,现在数据出现了不一致  , 这就是我们事务要解决的问题
             *
             * 事务: 他可以控制多个操作看成一个整体,  这个整体包含多个小的步骤 , 当中如果有一个小步骤失败了
             *  那么整个操作都失败, 数据要进行回滚 , 所有的步骤都成功, 才算成功
             */
            // 先转出
            // 把自动提交事务操作修改成手动提交
            connection.setAutoCommit(false);
            String outSQL = "update t_account set amount = amount-? where name= ?";
            preparedStatement = connection.prepareStatement(outSQL);
            preparedStatement.setBigDecimal(1,amount);
            preparedStatement.setString(2,fromName);
            preparedStatement.executeUpdate();

            // 在转入
            String intSQL = "update t_account set amount = amount+? where name= ?";
            preparedStatement = connection.prepareStatement(intSQL);
            preparedStatement.setBigDecimal(1,amount);
            preparedStatement.setString(2,toName);

//            int i =  1/0;    // 模拟断电的情况
            preparedStatement.executeUpdate();
            // 提交事务
            connection.commit();
        } catch (Exception e) {
            e.printStackTrace();
            // 回顾事务
            try {
                connection.rollback();
            } catch (SQLException ex) {
                ex.printStackTrace();
            }
        }finally {
            JDBCUtil.close(connection,null,preparedStatement);
        }

    }

    //根据用户名字查询用户余额是否足够   名字   amount
    @Override
    public boolean checkAccount(String name, BigDecimal amount) {
        // 获取连接对象
        Connection connection = JDBCUtil.getConnection();
        String sql = "select * from t_account where name = ? and amount >= ?";
        // 创建语句对象
        PreparedStatement preparedStatement = null;
        try {
             preparedStatement = connection.prepareStatement(sql);
             // 设置sql 参数
            preparedStatement.setString(1,name);
            preparedStatement.setBigDecimal(2,amount);
            // 执行sql
            ResultSet resultSet = preparedStatement.executeQuery();
            if(resultSet.next()){
                return true;
            }
        } catch (SQLException e) {
            e.printStackTrace();

        }finally {
            JDBCUtil.close(connection,null,preparedStatement);
        }
        return false;
    }
}
