package com.sf.day28.dao.impl;

import com.alibaba.druid.pool.DruidPooledConnection;
import com.sf.day28.dao.IUserDAO;
import com.sf.day28.domain.User;
import com.sf.day28.util.DruidUtil;
import com.sf.day28.util.JDBCUtil;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.Statement;
import java.util.ArrayList;
import java.util.List;

public class UserDAOImpl implements IUserDAO {
    @Override
    public void insert(User user) {
        // 加载驱动
        Connection connection =null;
        PreparedStatement preparedStatement =null;
        try {
            connection = JDBCUtil.getConnection();
            // 创建语句对象
            String sql = "insert into user(name,age,email,password) values(?,?,?,?)";
            preparedStatement = connection.prepareStatement(sql);
            preparedStatement.setString(1,user.getName());
            preparedStatement.setInt(2,user.getAge());
            preparedStatement.setString(3,user.getEmail());
            preparedStatement.setString(4,user.getPassword());
            // 执行sql语句
            preparedStatement.executeUpdate();
            // 关闭资源
        } catch (Exception e) {
            e.printStackTrace();
        }finally {
            JDBCUtil.close(connection,null,preparedStatement);
        }

    }

    @Override
    public void update(User user) {
        // 加载驱动
        Connection connection =null;
        PreparedStatement preparedStatement =null;
        try {
            connection = JDBCUtil.getConnection();
            // 创建语句对象
            String sql = "update user set name=?,age=?,email=?,password=?  where id = ?";
            preparedStatement = connection.prepareStatement(sql);
            preparedStatement.setString(1,user.getName());
            preparedStatement.setInt(2,user.getAge());
            preparedStatement.setString(3,user.getEmail());
            preparedStatement.setString(4,user.getPassword());
            preparedStatement.setLong(5,user.getId());
            // 执行sql语句
            preparedStatement.executeUpdate();
            // 关闭资源
        } catch (Exception e) {
            e.printStackTrace();
        }finally {
            JDBCUtil.close(connection,null,preparedStatement);
        }
    }

    @Override
    public void delete(Long id) {
        // 加载驱动
        Connection connection =null;
        PreparedStatement preparedStatement =null;
        try {
            connection = JDBCUtil.getConnection();
             // 创建语句对象
            String sql = "delete from user where id = ?";
            preparedStatement = connection.prepareStatement(sql);
            preparedStatement.setLong(1,id);
            // 执行sql语句
            preparedStatement.executeUpdate();
            // 关闭资源
        } catch (Exception e) {
            e.printStackTrace();
        }finally {
            JDBCUtil.close(connection,null,preparedStatement);
        }
    }

    @Override
    public User get(Long id) {
        // 加载驱动
        Connection connection =null;
        PreparedStatement preparedStatement =null;
        ResultSet resultSet = null;
        try {
            connection = JDBCUtil.getConnection();
            // 创建语句对象
            String sql = "select * from user where id = ?";
            preparedStatement = connection.prepareStatement(sql);
            preparedStatement.setLong(1,id);
            // 执行sql语句
            resultSet = preparedStatement.executeQuery();
            User user = new User();
            if(resultSet.next()){
                user.setName(resultSet.getString("name"));
                user.setPassword(resultSet.getString("password"));
                user.setEmail(resultSet.getString("email"));
                user.setId(resultSet.getLong("id"));
                user.setAge(resultSet.getInt("age"));
            }
            // 关闭资源
            return user;
        } catch (Exception e) {
            e.printStackTrace();
        }finally {
            JDBCUtil.close(connection,resultSet,preparedStatement);
        }
        return null;
    }

    @Override
    public List<User> list() {
        // 加载驱动
        DruidPooledConnection connection =null;
        PreparedStatement preparedStatement =null;
        ResultSet resultSet = null;
        try {

            // 通过德鲁伊连接池获取连接信息
            connection = DruidUtil.getConection();

            // 创建语句对象
            String sql = "select * from user";
            preparedStatement = connection.prepareStatement(sql);
            // 执行sql语句
            resultSet = preparedStatement.executeQuery();
            List<User> list = new ArrayList<>();
            while (resultSet.next()){
                User user = new User();
                user.setName(resultSet.getString("name"));
                user.setPassword(resultSet.getString("password"));
                user.setEmail(resultSet.getString("email"));
                user.setId(resultSet.getLong("id"));
                user.setAge(resultSet.getInt("age"));
                list.add(user);
            }
            // 关闭资源
            return list;
        } catch (Exception e) {
            e.printStackTrace();
        }finally {
            JDBCUtil.close(connection,resultSet,preparedStatement);
        }
        return null;
    }
}
