package com.sf.day28.util;

import java.io.InputStream;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.util.Properties;

public class JDBCUtil {
    // 加载驱动的这个操作只需要执行一次就可以了
    // 解决: 把加载驱动的操作放在静态代码块当中就可以解决了
    // 配置信息硬编码问题 , 连接数据库4要素, 我们现在是在代码写死了, 后续如果我们数据库密码要修改
    // 我们还需要重新修改java 代码
    // 解决: 把配置信息存储到配置文件当中 db.properties
    // 如何把配置文件当中信息, 设置对应位置中呢

    private static Properties properties = null;
    static {
        try {
            InputStream inputStream = Thread.currentThread().getContextClassLoader().getResourceAsStream("db.properties");
            properties = new Properties();
            // 调用load 方法加载我们属性信息
            properties.load(inputStream);
            // 加载驱动操作
            Class.forName(properties.getProperty("driverClassName"));
        } catch (Exception e) {
            e.printStackTrace();
        }
    }


    // 获取连接对象
    public static Connection getConnection(){
        Connection connection = null;
        try {
            // 获取连接对象
            connection = DriverManager.getConnection(properties.getProperty("url"),
                    properties.getProperty("username"),
                    properties.getProperty("password"));

        } catch (Exception e) {
            e.printStackTrace();
        }
        return connection;

    }

    // 要关闭资源
    // Connection PrepareStatemetn  Restset
    // 问题: 查询的时候需要关闭RestSet  , 增删改的时候不需要关闭RestSet
    public static void close(Connection connection , ResultSet resultSet, PreparedStatement preparedStatement){
        try{
            if (connection!=null){
                connection.close();
            }
        }catch (Exception e){
            e.printStackTrace();
        }
        try{
            if (preparedStatement!=null){
                preparedStatement.close();
            }
        }catch (Exception e){
            e.printStackTrace();
        }

        try{
            if (resultSet!=null){
                resultSet.close();
            }
        }catch (Exception e){
            e.printStackTrace();
        }
    }


}
